﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;

namespace TohleBudeUkol
{
    public class Board
    {
        private List<BoardCoords> nextPossiblePlays;
        private BoardCoords currentlySelected;
        int playerScore;
        int PlayerScore
        {
            set
            {
                playerScore = value;
                OnScoreChanged(value);
            }
            get
            {
                return playerScore;
            }
        }
        public event Action<int> OnScoreChanged = delegate { };
        public event Action<string> OnTimeChanged = delegate { };
        public event Action<Stone[,], BoardCoords, List<BoardCoords>> OnBoardRender = delegate { };
        /// <summary>
        /// row,column
        /// </summary>
        Stone[,] stonesData = new Stone[8, 8];
        StonesReserve stoneReserve;
        public Board(StreamReader reader)
        {
            nextPossiblePlays = new List<BoardCoords>();
            for (int baseRow = 0; baseRow < 8; baseRow++)
            {
                string lineString = reader.ReadLine();
                for (int baseColumn = 0; baseColumn < 8; baseColumn++)
                {
                    Stone stone = new Stone();
                    stone.Color = (StoneColor)int.Parse(lineString[baseColumn].ToString());
                    stonesData[baseRow, baseColumn] = stone;
                }
            }

            List<Stone>[] stoneReserveList = new List<Stone>[8];
            for (int reserveRow = 0; reserveRow < 8; reserveRow++)
            {
                stoneReserveList[reserveRow] = new List<Stone>();
                string lineString = reader.ReadLine();
                if (lineString != null)
                    foreach (char stoneNumberChar in lineString)
                    {
                        int stoneNumber = int.Parse(stoneNumberChar.ToString());
                        Stone stone = new Stone();
                        stone.Color = (StoneColor)stoneNumber;
                        stoneReserveList[reserveRow].Add(stone);
                    }
            }
            stoneReserve = new StonesReserve(stoneReserveList);
            render();
        }

        public void SelectAt(int row, int col)
        {
            BoardCoords clickedCoords = new BoardCoords(row, col);
            if (currentlySelected == null)
            {
                currentlySelected = clickedCoords;
                nextPossiblePlays.Clear();
                IEnumerable<Direction> values = Enum.GetValues(typeof(Direction)).Cast<Direction>();
                foreach (Direction direction in values)
                {
                    BoardCoords shifted = clickedCoords.Shift(direction);
                    if (canPlay(clickedCoords, shifted))
                        nextPossiblePlays.Add(shifted);
                }
            }
            else
            {
                int score = play(currentlySelected, clickedCoords);
                if (score > 0)
                {
                    PlayerScore += score;
                }
                currentlySelected = null;
            }
            render();
        }

        private int play(BoardCoords from, BoardCoords to)
        {
            if (Math.Abs(from.Row - to.Row) == 1 && Math.Abs(from.Col - to.Col) == 0 ||
                Math.Abs(from.Row - to.Row) == 0 && Math.Abs(from.Col - to.Col) == 1)
            {
                Stone[,] triedArray = (Stone[,])stonesData.Clone();
                swap(triedArray, from, to);
                int score = evaluateSleds(triedArray);
                if (score > 0)
                {
                    stonesData = triedArray;
                    int lastScore;
                    do
                    {
                        fillinEmpty();
                        lastScore = evaluateSleds(stonesData);
                    } while (lastScore > 0);
                    return score;
                }
                else
                    return score;
            }
            else
                return 0;
        }

        private BoardCoords canPlay(BoardCoords from)
        {
            IEnumerable<Direction> values = Enum.GetValues(typeof(Direction)).Cast<Direction>();
            foreach (Direction direction in values)
            {
                BoardCoords shifted = from.Shift(direction);
                if (canPlay(from, shifted))
                    return shifted;
            }
            return null;
        }

        private bool canPlay(BoardCoords from, BoardCoords to)
        {
            try
            {
                Stone[,] triedArray = (Stone[,])stonesData.Clone();
                swap(triedArray, from, to);
                int score = evaluateSleds(triedArray);
                return score > 0;
            }
            catch (IndexOutOfRangeException)
            {
                return false;
            }
        }

        public bool FindNextBest(out BoardCoords from, out BoardCoords to)
        {
            for (int row = 0; row < 8; row++)
                for (int col = 0; col < 8; col++)
                {
                    BoardCoords coords = new BoardCoords(row, col);
                    BoardCoords triedCoords = canPlay(coords);
                    if (triedCoords != null)
                    {
                        from = coords;
                        to = triedCoords;
                        return true;
                    }
                }
            from = null;
            to = null;
            return false;
        }

        private void swap(Stone[,] stones, BoardCoords from, BoardCoords to)
        {
            Stone fromStone = stones[from.Row, from.Col];
            stones[from.Row, from.Col] = stones[to.Row, to.Col];
            stones[to.Row, to.Col] = fromStone;
        }

        private int evaluateSleds(Stone[,] stones)
        {
            int score = 0;
            List<BoardCoords> sledStones = new List<BoardCoords>();
            for (int row = 0; row < 8; row++)
                for (int col = 0; col < 8; col++)
                {
                    BoardCoords coords = new BoardCoords(row, col);
                    Stone stone = getStoneAt(coords, stones);
                    List<Direction> directions = new List<Direction>();
                    directions.Add(Direction.up);
                    directions.Add(Direction.left);
                    foreach (Direction direction in directions)
                    {
                        BoardCoords coords1Ofsset = coords.Shift(direction);
                        Stone offset1Stone = getStoneAt(coords1Ofsset, stones);
                        if (offset1Stone != null && offset1Stone.Color == stone.Color)
                        {
                            BoardCoords coords2Ofsset = coords.Shift(DirectionUtil.GetOpposite(direction));
                            Stone offset2Stone = getStoneAt(coords2Ofsset, stones);
                            if (offset2Stone != null && offset2Stone.Color == stone.Color)
                            {
                                score += 3;
                                sledStones.AddIfNotContains(coords);
                                sledStones.AddIfNotContains(coords1Ofsset);
                                sledStones.AddIfNotContains(coords2Ofsset);
                            }
                        }
                    }

                }
            foreach (BoardCoords coords in sledStones)
            {
                stones[coords.Row, coords.Col] = null;
            }
            return score;
        }

        private void fillinEmpty()
        {
            for (int row = 7; row > -1; row--)
                for (int col = 0; col < 8; col++)
                {
                    BoardCoords coords = new BoardCoords(row, col);
                    if (getStoneAt(coords, stonesData) == null)
                    {
                        bool filled = false;
                        for (int rowUp = 0; rowUp <= row && !filled; rowUp++)
                        {
                            if (getStoneAt(coords.Up(rowUp), stonesData) != null)
                            {
                                stonesData[row, col] = stonesData[row - rowUp, col];
                                stonesData[row - rowUp, col] = null;
                                filled = true;
                            }
                        }
                        if (!filled)
                        {
                            stonesData[row, col] = stoneReserve.GetStoneFor(col);
                        }
                    }
                }
        }

        public static Stone getStoneAt(BoardCoords coords, Stone[,] stoness)
        {
            return getStoneAt(coords.Row, coords.Col, stoness);
        }

        public static Stone getStoneAt(int row, int col, Stone[,] stones)
        {
            if (row < 0 || col < 0 || row > 7 || col > 7)
                return null;
            else
                return stones[row, col];
        }

        public void ForceRender()
        {
            render();
        }

        private void render()
        {
            OnBoardRender(stonesData, currentlySelected, nextPossiblePlays);
        }
    }
}
