﻿using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace TohleBudeUkol
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        Board board;
        public MainWindow()
        {
            InitializeComponent();

            OpenFileDialog dialog = new OpenFileDialog();
            dialog.CheckFileExists = true;
            dialog.Title = "Vyberte soubor s daty desky";
            if ((bool)dialog.ShowDialog())
            { 
                board = new Board(new System.IO.StreamReader(dialog.FileName));
                board.OnScoreChanged += (score) => Dispatcher.Invoke(() => currentScore.Text = score.ToString());
                board.OnBoardRender += renderBoard;
                board.ForceRender();
            }

        }

        private void renderBoard(Stone[,] stonesData, BoardCoords currentlySelected, List<BoardCoords> possiblePlays)
        {
            mainCanvas.Children.Clear();
            for (int row = 0; row < 8; row++)
                for (int col = 0; col < 8; col++)
                {
                    Stone stone = Board.getStoneAt(row, col, stonesData);
                    if (stone != null)
                    {
                        Ellipse ellipse = new Ellipse();
                        ellipse.Fill = new SolidColorBrush(ColorUtil.GetRealColor(stone.Color));
                        ellipse.Width = Stone.DEFAULT_STONE_SIZE;
                        ellipse.Height = Stone.DEFAULT_STONE_SIZE;
                        Canvas.SetLeft(ellipse, col * Stone.DEFAULT_STONE_SIZE);
                        Canvas.SetTop(ellipse, row * Stone.DEFAULT_STONE_SIZE);
                        mainCanvas.Children.Add(ellipse);
                    }
                }
            if (currentlySelected != null)
            {
                Rectangle rect = new Rectangle();
                rect.Stroke = new SolidColorBrush(Colors.Orange);
                rect.StrokeThickness = 2;
                rect.Width = Stone.DEFAULT_STONE_SIZE;
                rect.Height = Stone.DEFAULT_STONE_SIZE;
                Canvas.SetLeft(rect, currentlySelected.Col * Stone.DEFAULT_STONE_SIZE);
                Canvas.SetTop(rect, currentlySelected.Row * Stone.DEFAULT_STONE_SIZE);
                mainCanvas.Children.Add(rect);

                foreach (BoardCoords coord in possiblePlays)
                {
                    Rectangle canPlayRect = new Rectangle();
                    canPlayRect.Stroke = new SolidColorBrush(Colors.ForestGreen);
                    canPlayRect.StrokeThickness = 2;
                    canPlayRect.Width = Stone.DEFAULT_STONE_SIZE;
                    canPlayRect.Height = Stone.DEFAULT_STONE_SIZE;
                    Canvas.SetLeft(canPlayRect, coord.Col * Stone.DEFAULT_STONE_SIZE);
                    Canvas.SetTop(canPlayRect, coord.Row * Stone.DEFAULT_STONE_SIZE);
                    mainCanvas.Children.Add(canPlayRect);
                }
            }
        }

        private void mainCanvas_MouseDown(object sender, MouseButtonEventArgs e)
        {
            Point point = e.GetPosition(mainCanvas);
            board.SelectAt((int)point.Y / Stone.DEFAULT_STONE_SIZE, (int)point.X / Stone.DEFAULT_STONE_SIZE);
        }
    }
}
